#!/usr/bin/perl -w
#
# quizz CGI script
#
# Thomas Linden <tom@co.daemon.de>
#
# Copyright (c) 2003 Thomas Linden.
# ALL RIGHTS RESERVED.
#
# Licensed under the terms of the artistic license.
#
# The module "Template" is Copyright 1999-2000 Jens Heunemann.
# Used with permission.
#


use CGI;
use Data::Dumper;
use CGI::Carp qw(fatalsToBrowser);
use Template;
use Config::General;
use DB_File;


# result database (dbm)
my %db;

# read/parse config file using Config::General
my %config = ParseConfig(
			 -ConfigFile       => "config/quizz.cfg",
			 -InterPolateVars  => 1,
			 -UseApacheInclude => 1,
			 -IncludeRelative  => 1
			   );

my %choice = (
	      M => "multiple",
	      S => "single",
	      R => "regex"
	      );

my @loop = @{$config{loop}};

# initialize stuff from config file
my $cgi      = "index.cgi";
my $VERSION  = "1.0.0";


# cgi object
my $query    = new CGI;

# initialize if not already done so far
&prepare;

# initialize shared templates
my $head     = new Template("templates/header.tpl");
my $foot     = new Template("templates/footer.tpl");

# initialize cgi parameters
my %input = map { $_ => $query->param($_) } $query->param;

# check it up where to go
my $sub2go   = $input{state} || "default";

# http
print "Content-Type: text/html\n\n\n";

# first the head
$head->print;

eval {
  # now do the real work
  &$sub2go;
};
if ($@) {
  print $@;
}
if ($config{debug}) {
  print "<pre>\n" . Dumper(\%input) . "\n</pre>";
}

# and the foot including the last changed date
my @stat = lstat($cgi);
$foot->code( date => scalar localtime( $stat[9] ) );
$foot->print;

exit;





###### all subs from now on, called by ::main via closure call (&$sub2go) #####
sub dump {
  my $var = shift;
  print "<pre>\n" . Dumper($var) . "\n</pre>";
}

sub default {
  my $T = new Template(-file => "templates/default.tpl");

  # compute best-of list
  my @_bestof;
  foreach my $user (keys %db) {
    my %entry = &get_entry($user);
    push @_bestof, \%entry;
  }
  my $num = 1;
  foreach my $entry (sort { $b->{points} <=>  $a->{points} } @_bestof) {
    if ($num <= $config{config}->{max_best}) {
      $entry->{pos} = $num;
      $entry->{color} = $loop[0];
      @loop = reverse @loop;
      push @bestof, $entry;
    }
    else {
      last;
    }
    $num++;
  }

  $T->code(
	   data => {
		    quiz_name        => $config{quiz_name},
		    quiz_description => $config{quiz_description},
		   },
	   bestof => \@bestof
	  );

  print $T->process;
}

sub quest {
  if ($input{user} eq "") {
    &default;
    return;
  }

  if (! exists $input{question}) {
    # starting point
    $input{question} = 1;
  }
  else {
    # previous run was a question
    my %q = %{ $config{question}->{ $input{question} } };
    my $factor = $q{points} / $q{time};
    my %entry = &get_entry($input{user});
    my ($points, $time, $result);

    if ($input{question} == 1) {
      $entry{points} = 0;
      $entry{time}   = 0;
    }

    # calculate time and points based on time
    # points will be zeroed if answer is wrong!
    if (exists $input{time}) {
      # time counter
      $points = ( int($input{time} * $factor) ) + $entry{points};
      $time   = $input{time} + $entry{time};
    }
    else {
      # point counter
      $points = $input{points} + $entry{points};
      $time   = ( int($input{points} / $factor) ) + $entry{time};
    }
    $result = "correct";

    # compute the answer
    if ($q{type} eq "M") {
      # multiple choice
      my %choices = map { $_ => 1 } $query->param("choice");
      my %correct = map { $_ => 1 } split /,/, $q{result};
      my $matches;
      foreach my $choice (keys %correct) {
	if (exists $choices{$choice}) {
	  $matches++;
	}
      }
      if ($matches != scalar keys %correct) {
	# shit, wrong answer!
	$points = $entry{points};
	$result = "wrong";
      }
    }
    elsif ($q{type} eq "S") {
      # single choice
      if ($input{choice} ne $q{result}) {
	# shit, wrong answer!
	$points = $entry{points};
	$result = "wrong";
      }
    }
    elsif ($q{type} eq "R") {
      # regex choice
      if ($input{choice} !~ /$q{result}/) {
	# shit, wrong answer!
	$points = $entry{points};
	$result = "wrong";
      }
    }

    print "<p><b>Your last answer was: $result</b></p>\n";

    &add_entry(time => $time, points => $points, user => $input{user}, date => scalar localtime(time));
    $input{question}++; # use next one
  }

  if (exists $config{question}->{ $input{question} }) {
    my $T = new Template(-file => "templates/question.tpl");
    my %question = %{ $config{question}->{ $input{question} } };

    if ($config{config}->{counter_time}) {
      $T->code(
	       timer => {
			 max      => $question{time},
			 question => $question{question}
			}
	      );
    }

    if ($config{config}->{counter_points}) {
      $T->code(
	       pointer => {
			   max      => $question{points},
			   interval => $question{points} / $question{time},
			   question => $question{question}
			  }
	      );
    }

    # prepare choices
    my @answers;
    if ($question{type} eq "R") {
      my %c = ( choice => "Answer:");
      push @answers, \%c;
    }
    else {
      foreach my $choice (sort {$a cmp $b } keys %{$question{choice}}) {
	my %c = (
		 choice => $choice,
		 answer => $question{choice}->{$choice}->{answer}
		);
	push @answers, \%c;
      }
    }

    $T->code(
	     $choice{ $question{type} } => \@answers,
	     form => { question => $input{question}, user => $input{user} }
	    );

    $T->code( data => { question => $input{question} } );

    print $T->process;
  }
  else {
    # no more questions, present the result
    # FIXME: add template!
    my %e = &get_entry($input{user});
    my $T = new Template(-file => "templates/result.tpl");
    $T->code( data => \%e );
    print $T->process;
  }
}








### helper subs ###
sub prepare {
  if (! -d "data/$config{key}") {
    system "mkdir data/$config{key}" and die "Could not prepare application!";
  }
  tie %db, "DB_File", "data/$config{key}/results.db" or die "Could not open database data/$config{key}/results.db: $!";
}

sub get_entry {
  my $user = shift;
  my ($points, $time, $date) = split /\//, $db{$user};
  my %entry = (
	       user   => $user,
	       date   => $date,
	       points => $points,
	       time   => $time
	      );
  return %entry;
}

sub add_entry {
  my %e = @_;
  $db{ $e{user} } = "$e{points}/$e{time}/$e{date}";
}
